@@ -33,6 +33,7 @@ from sentry_sdk.envelope import Envelope
 from sentry_sdk.profiler import has_profiling_enabled, setup_profiler
 from sentry_sdk.scrubber import EventScrubber
 from sentry_sdk.monitor import Monitor
+from sentry_sdk.spotlight import setup_spotlight
 
 from sentry_sdk._types import TYPE_CHECKING
 
@@ -236,11 +237,15 @@ class _Client(object):
             self.session_flusher = SessionFlusher(capture_func=_capture_envelope)
 
             self.metrics_aggregator = None  # type: Optional[MetricsAggregator]
-            if self.options.get("_experiments", {}).get("enable_metrics"):
+            experiments = self.options.get("_experiments", {})
+            if experiments.get("enable_metrics"):
                 from sentry_sdk.metrics import MetricsAggregator
 
                 self.metrics_aggregator = MetricsAggregator(
-                    capture_func=_capture_envelope
+                    capture_func=_capture_envelope,
+                    enable_code_locations=bool(
+                        experiments.get("metric_code_locations")
+                    ),
                 )
 
             max_request_body_size = ("always", "never", "small", "medium")
@@ -268,6 +273,10 @@ class _Client(object):
                 ],
             )
 
+            self.spotlight = None
+            if self.options.get("spotlight"):
+                self.spotlight = setup_spotlight(self.options)
+
             sdk_name = get_sdk_name(list(self.integrations.keys()))
             SDK_INFO["name"] = sdk_name
             logger.debug("Setting SDK name to '%s'", sdk_name)
@@ -548,8 +557,6 @@ class _Client(object):
         if disable_capture_event.get(False):
             return None
 
-        if self.transport is None:
-            return None
         if hint is None:
             hint = {}
         event_id = event.get("event_id")
@@ -591,7 +598,11 @@ class _Client(object):
         # If tracing is enabled all events should go to /envelope endpoint.
         # If no tracing is enabled only transactions, events with attachments, and checkins should go to the /envelope endpoint.
         should_use_envelope_endpoint = (
-            tracing_enabled or is_transaction or is_checkin or bool(attachments)
+            tracing_enabled
+            or is_transaction
+            or is_checkin
+            or bool(attachments)
+            or bool(self.spotlight)
         )
         if should_use_envelope_endpoint:
             headers = {
@@ -616,9 +627,18 @@ class _Client(object):
             for attachment in attachments or ():
                 envelope.add_item(attachment.to_envelope_item())
 
+            if self.spotlight:
+                self.spotlight.capture_envelope(envelope)
+
+            if self.transport is None:
+                return None
+
             self.transport.capture_envelope(envelope)
 
         else:
+            if self.transport is None:
+                return None
+
             # All other events go to the legacy /store/ endpoint (will be removed in the future).
             self.transport.capture_event(event_opt)
 
