@@ -102,6 +102,7 @@ class Span(object):
         "hub",
         "_context_manager_state",
         "_containing_transaction",
+        "_local_aggregator",
     )
 
     def __new__(cls, **kwargs):
@@ -162,6 +163,7 @@ class Span(object):
         self.timestamp = None  # type: Optional[datetime]
 
         self._span_recorder = None  # type: Optional[_SpanRecorder]
+        self._local_aggregator = None  # type: Optional[LocalAggregator]
 
     # TODO this should really live on the Transaction class rather than the Span
     # class
@@ -170,6 +172,13 @@ class Span(object):
         if self._span_recorder is None:
             self._span_recorder = _SpanRecorder(maxlen)
 
+    def _get_local_aggregator(self):
+        # type: (...) -> LocalAggregator
+        rv = self._local_aggregator
+        if rv is None:
+            rv = self._local_aggregator = LocalAggregator()
+        return rv
+
     def __repr__(self):
         # type: () -> str
         return (
@@ -479,6 +488,8 @@ class Span(object):
             self.timestamp = datetime_utcnow()
 
         maybe_create_breadcrumbs_from_span(hub, self)
+        add_additional_span_data(hub, self)
+
         return None
 
     def to_json(self):
@@ -499,6 +510,11 @@ class Span(object):
         if self.status:
             self._tags["status"] = self.status
 
+        if self._local_aggregator is not None:
+            metrics_summary = self._local_aggregator.to_json()
+            if metrics_summary:
+                rv["_metrics_summary"] = metrics_summary
+
         tags = self._tags
         if tags:
             rv["tags"] = tags
@@ -722,6 +738,13 @@ class Transaction(Span):
 
         event["measurements"] = self._measurements
 
+        # This is here since `to_json` is not invoked.  This really should
+        # be gone when we switch to onlyspans.
+        if self._local_aggregator is not None:
+            metrics_summary = self._local_aggregator.to_json()
+            if metrics_summary:
+                event["_metrics_summary"] = metrics_summary
+
         return hub.capture_event(event)
 
     def set_measurement(self, name, value, unit=""):
@@ -998,7 +1021,9 @@ def trace(func=None):
 from sentry_sdk.tracing_utils import (
     Baggage,
     EnvironHeaders,
+    add_additional_span_data,
     extract_sentrytrace_data,
     has_tracing_enabled,
     maybe_create_breadcrumbs_from_span,
 )
+from sentry_sdk.metrics import LocalAggregator
